#ifndef AREA_HPP_INCLUDED
#define AREA_HPP_INCLUDED

#include "Tile.hpp"
#include "Convex.hpp"
#include <vector>
#include <cstdint>

/** @addtogroup Types
* @{ */
class Area : public Convex
{
    public:
		/**
		* @brief The Plane the Area is located on
		* @par Description
		* The Plane the Area is located on
		*/
        std::int32_t Plane;

        /**
        * @brief Default Constructor, constructs an Area with -1 as the TopLeft and BottomRight Tile
        * @par Description
        *   Default Constructor, constructs an Area with -1 as the TopLeft and BottomRight Tile
        */
        Area();
        /**
        * @brief Constructs an Area based on a TopLeft tile and a BottomRight tile, sets the internal Plane to the plane of the TopLeft tile
		* @param Tile TopLeft    Top Left tile of the Area
		* @param Tile BottomRight    Bottom Right tile of the Area
		*/
        Area(Tile TopLeft, Tile BottomRight);
        /**
        * @brief Constructs an Area based on a TopLeft tile and a BottomRight tile, sets the internal Plane to the plane of the TopLeft tile
		* @param std::vector<Tile> TileVertices
		*/
        Area(std::vector<Tile> TileVertices);
        /**
        * @brief Constructs an Area, by copying another
		* @param const Area& A  The Area to "copy"
		*/
        Area(const Area& A);

        /**
		* @brief Returns True if the Area contains a Tile, does check the Tiles Plane
		* @return True if the Area contains a Tile, does check the Tiles Plane
		* @par Example
		* @code
		*   Area A(Tile(100, 100, 0), Tile(200, 200, 0));
		*   Debug::Info << A.contains(Tile(150, 150, 0));
		* @endcode
		*/
        bool Contains(const Tile& T) const;
        /**
		* @brief Returns the middle Tile of the Area
		* @return The middle Tile of the Area
		* @par Example
		* @code
		*   Area A(Tile(100, 100, 0), Tile(200, 200, 0));
		*   Debug::Info << A.GetMiddle());
		* @endcode
		*/
        Tile GetMiddle() const;
        /**
		* @brief Returns a random Tile using Hybrid Distribution
		* @return A random Tile using Hybrid Distribution
		* @par Example
		* @code
		*   Area A(Tile(100, 100, 0), Tile(200, 200, 0));
		*   Debug::Info << A.GetHybridRandomTile(0.20, 0.20, 0.50, false));
		* @endcode
		* @see @ref HybridRandomDocs
		* @see @ref MissChanceFollowupDocs
		*/
        Tile GetHybridRandomTile(double ProbabilityX, double ProbabilityY, double StandardDeviationX, double StandardDeviationY, bool CheckMissFollowupChance = true) const;

        friend std::ostream& operator<<(std::ostream& OS, const Area& A);

        ~Area();
};

/** @} */

#endif // AREA_HPP_INCLUDED
